@import url(code.css);
@import url(changeset.css);
@import url(text-diff-html.css);

/* Browser */
h1 { font-size: 16px; letter-spacing: -0.028em; margin: 0; padding: 0 0 .5em }
h1 :link, h1 :visited, h1 .filename { border: none; padding: 0 .2em }
h1 :link, h1 :visited { color: #b00 }
h1 .first:link, h1 .first:visited { color: #998 }
h1 .sep { color: #666; padding: 0 .1em }

#navpath { float: left; }

#language-selection { float: right; font-size: 10px; }
#language-selection select { font-size: 10px; }
#language-selection input { font-size: 10px; }

#content { clear: both; }

#jumprev { float: right; font-size: 10px; margin: 0 0 0 }
#jumprev form { margin: 0 }
#jumprev input { font-size: 10px; margin-right: 0 }

/* Styles for the directory entries table
   (extends the styles for "table.listing") */
#dirlist { margin-top: 0 }
#dirlist td.rev, #dirlist td.age, #dirlist td.change, #dirlist td.size {
 color: #888;
 white-space: nowrap;
}
#dirlist td.size { text-align: right; }
#dirlist td.name { width: 100% }
#dirlist td.name a, #dirlist td.name span {
 background-position: 0% 50%;
 background-repeat: no-repeat;
 padding-left: 20px;
}
#dirlist td.name a.parent { background-image: url(parent.png) }
#dirlist td.name a.dir { background-image: url(folder.png) }
#dirlist td.name span.dir { background-image: url(folderdeny.png) }
#dirlist td.name a.file { background-image: url(file.png) }
#dirlist td.name span.file { background-image: url(filedeny.png) }
#dirlist td.name a, #dirlist td.rev a { border-bottom: none; display: block }
#dirlist td.change * { font-size: 9px }

/* Styles for the property list table
   (extends the styles for "table.listing") */
#proplist td.propvalue { width: 100%; }

/* Styles for the revision log table
   (extends the styles for "table.listing") */
#chglist { margin-top: 0 }
#chglist td.date, #chglist td.diff { white-space: nowrap }
#chglist td.rev, #chglist td.chgset { text-align: center }
#chglist td.rev a, #chglist td.chgset a { border-bottom: none }
#chglist td.summary { width: 100%; }

/* Styles for the revision info in the file module */
#info {
 background: #f7f7f0;
 border: 1px solid #d7d7d7;
 border-collapse: collapse;
 border-spacing: 0;
 clear: both;
 width: 100%;
}
#info th {
 font-size: 11px;
 font-weight: bold;
 padding: 2px .5em;
 white-space: nowrap;
 vertical-align: top;
}
#info td.message {
 font-size: 11px;
 padding: 2px .5em;
 vertical-align: top;
 width: 100%;
}
#info .message ul { padding: 0; margin: 0 2em }
#info .message p { margin: 0; padding: 0 }

/* Styles for the HTML preview */
#preview { background: #fff; clear: both; margin: 0 }
#preview .code-block { border-top: 1px solid #999; margin: 0 }
#preview .image-file { overflow: hidden }
#preview .image-file img { max-width: 100% }
