.file span { 
  display: block;
}
.file .fileheader, .file .hunkheader {
  color: #888;
}
.file .hunk { 
  font-family: monospace;
  white-space: pre;
}
.file .hunk .ctx {
  background: #fff;
}
.file .hunk .ins { 
  background: #dfd; 
  text-decoration: none; 
  display: block; 
}
.file .hunk .del {
  background: #fdd;
  text-decoration: none;
  display: block;
}
.ins { 
  border: 1px solid #0c0;
}
.del { 
  border: 1px solid #c00;
}
span.diff-leader {
  display: none;
}
